#include-once

Global Const $MAX_FIELDS = 256
Global Const $MAX_BUFFER_SIZE = 65534
Global Enum $FIELD_TYPE, $FIELD_NAME, $FIELD_UNIT

Global Const $ft_nomorefields = 0 
Global Const $ft_numeric_32 = 1 
Global Const $ft_numeric_64 = 2 
Global Const $ft_numeric_floating = 3 
Global Const $ft_date = 4 
Global Const $ft_time = 5 
Global Const $ft_boolean = 6 
Global Const $ft_multiplechoice = 7 
Global Const $ft_string = 8 
Global Const $ft_fulltext = 9 
Global Const $ft_datetime = 10 
Global Const $ft_stringw = 11 

Global Const $ft_nosuchfield = -1
Global Const $ft_fileerror = -2
Global Const $ft_fieldempty = -3
Global Const $ft_ondemand = -4

Global Const $tagContentDefaultParamStruct = "int;dword;dword;char[256]"

Func _PHWDXPluginOpen($sPlugin)
    Local $hPlugin = DllOpen($sPlugin)
    If @Error Then Return SetError(1,0,"")
    Local $pDPS = DllStructCreate($tagContentDefaultParamStruct)
    ; This stupid of course, it's need an actual data
    DllStructSetData($pDPS, 1, 272)
    DllStructSetData($pDPS, 2, 10)
    DllStructSetData($pDPS, 3, 2)
    DllStructSetData($pDPS, 4, EnvGet("COMMANDER_PATH") & "\contplug.ini")
    DllCall($hPlugin, "none", "ContentSetDefaultParams", "ptr", DllStructGetPtr($pDPS))
    Return $hPlugin
EndFunc

Func _PHWDXPluginClose($hPlugin)
    DllCall($hPlugin, "none", "ContentPluginUnloading")
    DllClose($hPlugin)
EndFunc

Func _PHWDXGetFieldIndexByName($vPlugin, $sFieldName)
    Local $aFields = _PHWDXGetFields($vPlugin)
    If @error Then Return SetError(1,0,-1)
    For $i = 0 To UBound($aFields)-1 
        If $sFieldName = $aFields[$i][1] Then Return $i
    Next
    Return SetError(2,0,-1)
EndFunc

Func _PHWDXGetUnitIndexByName($vPlugin, $sFieldName, $sUnitName)
    Local $aFields = _PHWDXGetFields($vPlugin)
    If @error Then Return SetError(1,0,-1)
    For $i = 0 To UBound($aFields)-1 
        If $sFieldName = $aFields[$i][$FIELD_NAME] Then 
            $aUnits = StringSplit($aFields[$i][$FIELD_UNIT], "|")
            For $j = 1 To $aUnits[0]
                If $aUnits[$j] = $sUnitName Then 
                     Return $j
                EndIf
            Next
            Return 0
        EndIf
    Next
    Return SetError(2,0,-1)
EndFunc

Func _PHWDXGetFields($vPlugin)
    Local $aFields[$MAX_FIELDS][3], $nFiledCount = 0
    For $i = 0 To $MAX_FIELDS - 1 
        $aRet = DllCall($vPlugin, "int", "ContentGetSupportedField", "int", $i, "str", "", "str", "", "int", $MAX_BUFFER_SIZE) 
        If @error Then ExitLoop 
        If $aRet[0] = $ft_nomorefields Then ExitLoop
        $aFields[$i][$FIELD_TYPE] = $aRet[0]
        $aFields[$i][$FIELD_NAME] = $aRet[2]
        $aFields[$i][$FIELD_UNIT] = $aRet[3]
        $nFiledCount += 1
    Next
    If $nFiledCount = 0 Then Return SetError(1,0,"")
    ReDim $aFields[$nFiledCount][3]
    Return $aFields
EndFunc

Func _PHWDXGetValues($vPlugin, $sFileName)
    Local $aFieldValues[$MAX_FIELDS], $nFieldCount = 0
    For $i = 0 To $MAX_FIELDS - 1
        $vFieldValue = __PHWDXGetFieldValue($vPlugin, $sFileName, $i, 0)
        If @error Then ExitLoop
        $aFieldValues[$i] = $vFieldValue
        $nFieldCount += 1
    Next
    
    If $nFieldCount > 0 Then 
        ReDim $aFieldValues[$nFieldCount]
        Return $aFieldValues
    Else
        Return SetError(1, 0, "")
    EndIf
EndFunc

Func _PHWDXGetValue($vPlugin, $sFileName, $nFieldIdx, $nUnitIdx = 0)
    Local $vFieldValue = __PHWDXGetFieldValue($vPlugin, $sFileName, $nFieldIdx, $nUnitIdx)
    If @error Then
        Return SetError(@error, 0, "")
    Else
        Return $vFieldValue
    EndIf
EndFunc

Func __PHWDXGetFieldValue($vPlugin, $sFileName, $nFieldIdx, $nUnitIdx)
    Local $pBuffer = DllStructCreate("byte[" & $MAX_BUFFER_SIZE & "]")
    $aRet = DllCall($vPlugin, "int", "ContentGetValueW", "wstr", $sFileName, "int", $nFieldIdx, "int", $nUnitIdx, "ptr", DllStructGetPtr($pBuffer), "int", $MAX_BUFFER_SIZE, "int", 0)
    ; in case of error "function not found", try to load ANSI version
    If @error = 3 Then $aRet = DllCall($vPlugin, "int", "ContentGetValue", "str", $sFileName, "int", $nFieldIdx, "int", $nUnitIdx, "ptr", DllStructGetPtr($pBuffer), "int", $MAX_BUFFER_SIZE, "int", 0) 
    If @error Then Return SetError(1, 0, "")
    Switch $aRet[0]
        Case $ft_numeric_32 
            $pValue = DllStructCreate("int", DllStructGetPtr($pBuffer)) 
        Case $ft_numeric_64 
            $pValue = DllStructCreate("int64", DllStructGetPtr($pBuffer)) 
        Case $ft_numeric_floating 
            $pValue = DllStructCreate("double", DllStructGetPtr($pBuffer)) 
        Case $ft_string, $ft_fulltext 
            $pValue = DllStructCreate("char[" & $MAX_BUFFER_SIZE & "]", DllStructGetPtr($pBuffer)) 
        Case $ft_stringw 
            $pValue = DllStructCreate("wchar[" & $MAX_BUFFER_SIZE / 2 & "]", DllStructGetPtr($pBuffer))
        Case $ft_date
            $pDate = DllStructCreate("word;word;word", DllStructGetPtr($pBuffer))
            Return StringFormat("%04d-%02d-%02d", _ 
                                DllStructGetData($pDate, 1), _ 
                                DllStructGetData($pDate, 2), _ 
                                DllStructGetData($pDate, 3))
        Case $ft_time
            $pTime = DllStructCreate("word;word;word", DllStructGetPtr($pBuffer))
            Return StringFormat("%02d:%02d:%02d", _ 
                                DllStructGetData($pTime, 1), _ 
                                DllStructGetData($pTime, 2), _ 
                                DllStructGetData($pTime, 3))
        Case $ft_datetime
            $pFileTime = DllStructCreate("dword;dword", DllStructGetPtr($pBuffer)) 
            $pLocalTime = DllStructCreate("dword;dword") 
            DllCall("Kernel32.dll", "int", "FileTimeToLocalFileTime", "ptr", DllStructGetPtr($pFileTime), "ptr", DllStructGetPtr($pLocalTime)) 
            $pValue = DllStructCreate("short;short;short;short;short;short;short;short") 
            DllCall("Kernel32.dll", "int", "FileTimeToSystemTime", "ptr", DllStructGetPtr($pLocalTime), "ptr", DllStructGetPtr($pValue)) 
            Return StringFormat("%04d-%02d-%02d %02d:%02d:%02d", _ 
                                DllStructGetData($pValue, 1), _ 
                                DllStructGetData($pValue, 2), _ 
                                DllStructGetData($pValue, 4), _ 
                                DllStructGetData($pValue, 5), _ 
                                DllStructGetData($pValue, 6), _ 
                                DllStructGetData($pValue, 7))
        Case $ft_nosuchfield, $ft_fileerror, $ft_fieldempty
            Return SetError(2, 0, "")
        Case Else
            Return SetError(3, 0, "")
    EndSwitch 
    Return DllStructGetData($pValue, 1)
EndFunc

